/*** INCLUDES ***/
#define F_CPU 16000000
//#include <mega32.h>
#include <avr/io.h>
#include <stdio.h>
#include <util/delay.h>
#include <math.h>
#include <avr\interrupt.h>
/*** DEFINES ***/

/*** PROTOTYPES ***/
int uart_putchar(char c, FILE *unused);
int uart_getchar(FILE* f);
FILE mystdout = FDEV_SETUP_STREAM(uart_putchar, NULL, _FDEV_SETUP_WRITE);
FILE mystdin =  FDEV_SETUP_STREAM(NULL, uart_getchar, _FDEV_SETUP_READ);
void initialize();
/*** GLOBALS ***/
char val;
// current measurement state
enum {PULSE, GSR, BREATH} measure, nextmeasure;
// current signal voltages
char pulseV = 0;
char gsrV = 0;
char breathV = 0;
// enter every 100Hz
void getAD(void)
{	// sample A/D and store
	ADCSRA |= 1<<ADSC;
	while(ADCSRA & (1<<ADSC));
		val = ADCH;
	// start a/d converter
	switch(measure)
	{
		case PULSE:
			pulseV = val;
			nextmeasure = GSR;
			break;
		case GSR:
			gsrV = val;
			nextmeasure = BREATH;
			break;
		case BREATH:
			breathV = val;
			nextmeasure = PULSE;
			break;
	}
	switch(nextmeasure)
	{
		case PULSE:  ADMUX = 0b01100001; break;
		case GSR:    ADMUX = 0b01100010; break;
		case BREATH: ADMUX = 0b01100011; break;
	}
	measure = nextmeasure;
	//ADCSRA = (1<<ADSC);
}
ISR(TIMER0_COMP_vect) {
	//PORTD ^= 0x80;
	getAD();
}// send signals to MATLAB for drawing

void transmit()
{
	printf("%d ", (int)pulseV);
	printf("%d ", (int)gsrV);
	printf("%d ", (int)breathV);
	printf("\r");	// end packet
	PORTD ^= (1<<PD7);
}
// read A/D converter and communicate with MATLAB
int main(void)
{
	char inchar;
	initialize();
	while(1)
	{
		getAD();
		// when signaled by MATLAB, return current values
		if (UCSRA & (1<<7))
		{
			inchar = UDR;
			if (inchar=='s') {
				transmit();
			}
		}
	}
	printf("Exit from main\n");
	return 0;
}// setup
void initialize(void)
{
	// comm indicator LED
	DDRD |= (1<<PD7);
	PORTD &= ~(1<<PD7);
	// serial RS-232  setup for debugging using printf, etc.
	//UCSRB = 0x18;
	UCSRB = (1<<RXEN)|(1<<TXEN);
	//UCSRC = (1<<URSEL);
	UCSRC = (1<<UCSZ0) | (1<<UCSZ1) |  (1<<URSEL);
	UBRRL = 103;
	UBRRH = 0;
	stdin = &mystdin;
	stdout = &mystdout;
		// set up timer0 to sample a/d at about 100Hz
	//TCCR0 = 0b00000101;
	//TIMSK = 0b00000010;
	//OCR0 = 156;
	/*OCR1A = 15624;
	TCCR1A = 0;
	TCCR1B = (1<<WGM12)|(1<<CS12)|(1<<CS10);
	TIMSK |= _BV(OCIE1B);
	// set up a/d for external Vref, channel 0
	// channel zero / left adj / AVcc Reference
	// A1=PULSE, A2=GSR, A3=BREATH
	*/ADMUX = 0b01100001;
	// enable ADC and set prescaler to 1/128*16MHz=125kHz
	// and clear interupt enable
	// and start a conversion
	ADCSRA = 0b10000111;
	measure = PULSE;
	// and start the show
	//sei();
}
int uart_putchar(char c, FILE *unused)      
{	if (c == '\n')
		uart_putchar('\r', 0);
	loop_until_bit_is_set(UCSRA, UDRE);
	UDR = c;
	return 0;
}// Functie ce trateaza primirea unui caracter pe seriala
int uart_getchar(FILE* f)
{	char c;
    
    loop_until_bit_is_set(UCSRA, RXC);
    c = UDR;
    if(c=='\r')
		c='\n';
    uart_putchar(c,NULL); 
    return c;
}