% CLOSE LEFTOVER SERIAL CONNECTIONS
clear all
try
	fclose(instrfind) %close any bogus serial connections 
end


% SET UP SERIAL CONNECTION THROUGH RS-232
% set its rate to 9600 baud
% SR830 terminator character is carriage return (ACSII 13)

s = serial('COM2', 'baudrate', 9600, 'terminator', 13);
fopen(s);
disp('serial communication setup complete');


% CONSTANTS
WARMUP_TIME = 200;
PULSE_WINDOW_SIZE = 100;
BREATH_WINDOW_SIZE = 300;
PLOT_SIZE = 1000;
PDELTA = 0.25;
BDELTA = 0.25;

vect = zeros(1000,3);

% SET UP PLOTS AND GRAB HANDLES
figure(1);

subplot(3, 1, 1);
hPulse = plot(vect(:,1), 'r');
hold on;
hpPeaks = scatter(vect(:,1), vect(:,2));
hold off;
axis([1 1000 1 4]);
hPulseTitle = title('Pulse');

subplot(3, 1, 2);
hGSR = plot(vect(:,2), 'g');
%axis([1 1000 0 2.5]);
hGSRTitle = title('GSR');

subplot(3, 1, 3);
hBreath = plot(vect(:,3), 'b');
hold on;
hbPeaks = scatter(vect(:,1), vect(:,2),'r');
hold off;
axis([1 1000 0 5]);
hBreathTitle = title('Breath');

set(hPulse,		'Erase', 'xor');
set(hpPeaks,	'Erase', 'xor');
set(hbPeaks,	'Erase', 'xor');
set(hGSR,		'Erase', 'xor');
set(hBreath,	'Erase', 'xor');

% ENTER INFINITE PLOTTING LOOP
i = 1;
j = 1;
k = 1;
time_100 = 1;
lbRate = 1;
lpRate = 1;
init_cnt = 1;
tempP = (1:2);
tempG = (1:2);
tempB = (1:2);

pfilt = [0.0039 0.0313 0.1094 0.2188 0.2734 0.2188 0.1094 0.0313 0.0039]; % approximate gaussian filter for pulse signal
bfilt = ones(1,10) .* 0.1; % uniform filter for breath signal
while (1)
	% SIGNAL MCU THAT WE WANT VALUES
	fprintf(s,'s');
	% READ VALUES FROM MCU
	vect(i,:) = fscanf(s,'%d %d %d');    
    vect;

	% FILTER SIGNALS FOR SMOOTHING
	filtPulse = conv(vect(:,1), pfilt);
	filtGSR = conv(vect(:,2), pfilt);
	filtBreath = conv(vect(:,3), bfilt);

	% UPDATE SIGNAL PLOTS

	set(hPulse, 'YData', filtPulse);
	set(hGSR, 'YData', filtGSR(10:1000));
	set(hBreath, 'YData', filtBreath);
	drawnow;

	% COMPUTE BREATHING AND PULSE RATES & INSTANTANEOUS GSR
	if (init_cnt > WARMUP_TIME)
		% COMPUTE INSTANTANEOUS GSR
		Inst_GSR = 1/((5 - filtGSR(i))/(1000000*filtGSR(i)) - 1/(1000000));

		% CALCULATE AVERAGE BREATH RATE OVER ENTIRE PLOT
		bPeaks = peakdet(filtBreath, BDELTA);
		nbPeaks = size(bPeaks);
		temp2 = bPeaks(2:nbPeaks);

		bRate = mean(temp2' - bPeaks(1:nbPeaks-1,1));
		bSTD = std(temp2' - bPeaks(1:nbPeaks-1,1));

		lfiltBreath(k) = filtBreath(i);

		% COMPUTE AND DISPLAY INSTANTANEOUS BREATH RATE
		if(k > BREATH_WINDOW_SIZE)
			lbPeaks = peakdet(lfiltBreath, BDELTA);
			lnbPeaks = size(lbPeaks);
			ltemp2 = lbPeaks(2:lnbPeaks);
			if(size(ltemp2,1) ~= 0 && size(lbPeaks,1) ~= 0)
				lbRate = mean(ltemp2' - lbPeaks(1:lnbPeaks-1,1));
			end
			Inst_Breath = (60/lbRate)*(BREATH_WINDOW_SIZE/(time_100*(BREATH_WINDOW_SIZE/PULSE_WINDOW_SIZE))); %scaling function
			bTemp = (60/bRate)*(BREATH_WINDOW_SIZE/(time_100*(BREATH_WINDOW_SIZE/PULSE_WINDOW_SIZE))); %scaling function

			bTitle = sprintf('Breath (Current: %4.2f [Breaths/Min], Mean: %4.2f, SD: %4.2f)', Inst_Breath, bTemp, bSTD);
			set(hBreathTitle, 'String', bTitle);

			set(hbPeaks, 'XData', bPeaks(:,1));
			set(hbPeaks, 'YData', bPeaks(:,2));

			k = 1;
		end
		k = k + 1;

		% CALCULATE AVERAGE PULSE RATE
		pPeaks = peakdet(filtPulse, PDELTA); % (x,y) values of peaks in pulse signal
		npPeaks = size(pPeaks);
		temp = pPeaks(2:npPeaks);

		pRate = mean(temp' - pPeaks(1:npPeaks-1,1)); % average Pulse Rate
		pSTD = std(temp' - pPeaks(1:npPeaks-1,1)); % STD of Pulse Rate

		lfiltPulse(j) = filtPulse(i);

		% COMPUTE AND DISPLAY PULSE RATE
		if (j > PULSE_WINDOW_SIZE)
			lpPeaks = peakdet(lfiltPulse, PDELTA); % (x,y) values of local peaks
			lnpPeaks = size(lpPeaks); % number of local peaks
			ltemp = lpPeaks(2:lnpPeaks);
			if (size(ltemp,1) ~= 0 && size(lpPeaks,1) ~= 0)
				lpRate = mean(ltemp' - lpPeaks(1:lnpPeaks-1,1));
			end
			Inst_Pulse = (60/lpRate)*(PULSE_WINDOW_SIZE/time_100); % instantaneous pulse rate

			set(hpPeaks, 'XData', pPeaks(:,1));
			set(hpPeaks, 'YData', pPeaks(:,2));

			pTemp = (60/pRate)*(PULSE_WINDOW_SIZE/time_100); % scaling function
			pTemp2 = (60/pSTD)*(PULSE_WINDOW_SIZE/time_100); % scaling function

			%CALCULATES INSTANTANEOUS GSR
			gTemp = mean(filtGSR(10:1000));
			gTemp2 = std(filtGSR(10:1000));

			gAve = 1/((5 - gTemp)/(1000000*gTemp) - 1/(1000000));
			gSTD = 1/((5 - gTemp2)/(1000000*gTemp2) - 1/(1000000));

			%DISPLAYS READING INFO FOR ALL READINGS
			gTitle = sprintf('GSR (Current: %6.2f [Kilo Ohms], Mean: %4.2f, SD: %4.2f)', Inst_GSR/1000, gAve/1000, gSTD/1000);
			set(hGSRTitle, 'String', gTitle);

			pTitle = sprintf('Pulse (Current: %5.2f [Beats/Min], Mean: %4.2f, SD: %4.2f)', Inst_Pulse, pTemp, pSTD);
			set(hPulseTitle, 'String', pTitle);

			j = 1;
		end

		if(j == 1) 
			tic;
		end
		if(j == PULSE_WINDOW_SIZE)
			time_100 = toc;
		end
		j = j + 1;
	end
	if (i == PLOT_SIZE + 1)
		i = 1;
	end

	init_cnt = init_cnt + 1;
	i = i + 1;
end

% CLOSE THE SERIAL CONNECTION (UNFORTUNATELY WE NEVER GET HERE)
fclose(s);

